package cn.iocoder.yudao.module.customer.service.customer.commission;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.customer.convert.customer.commission.CustomerCommissionInfoConvert;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.commission.CustomerCommissionDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.commission.CustomerCommissionInfoDO;
import cn.iocoder.yudao.module.customer.dal.mysql.customer.commission.CustomerCommissionInfoMapper;
import cn.iocoder.yudao.module.customer.dal.mysql.customer.commission.CustomerCommissionMapper;
import cn.iocoder.yudao.module.customer.dto.CustomerCommissionDto;
import cn.iocoder.yudao.module.customer.dto.CustomerCommissionQueryDto;
import cn.iocoder.yudao.module.customer.enums.CommissionTypeEnum;
import cn.iocoder.yudao.module.customer.vo.customer.commission.CustomerCommissionInfoCreateReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.commission.CustomerCommissionInfoExportReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.commission.CustomerCommissionInfoPageReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.commission.CustomerCommissionInfoUpdateReqVO;
import cn.iocoder.yudao.module.depository.dal.mysql.warehouse.WarehouseLineMapper;
import cn.iocoder.yudao.module.depository.dto.LogisticsInfoDto;
import cn.iocoder.yudao.module.product.ProductApi;
import cn.iocoder.yudao.module.product.vo.product.ProductRespVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.CUSTOMER_COMMISSION_INFO_NOT_EXISTS;

/**
 * 客户佣金详情 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class CustomerCommissionInfoServiceImpl extends AbstractService<CustomerCommissionInfoMapper, CustomerCommissionInfoDO> implements CustomerCommissionInfoService {

    @Resource
    private CustomerCommissionInfoMapper customerCommissionInfoMapper;

    @Resource
    private CustomerCommissionMapper customerCommissionMapper;

    @Resource
    private ProductApi productApi;

    @Resource
    private WarehouseLineMapper warehouseLineMapper;

    private static final Integer ALL_INTEGER = -1;
    private static final Long ALL_LONG = -1L;


    @Override
    public Long createCustomerCommissionInfo(CustomerCommissionInfoCreateReqVO createReqVO) {
        // 插入
        CustomerCommissionInfoDO customerCommissionInfo = CustomerCommissionInfoConvert.INSTANCE.convert(createReqVO);
        customerCommissionInfoMapper.insert(customerCommissionInfo);
        // 返回
        return customerCommissionInfo.getId();
    }

    @Override
    public void updateCustomerCommissionInfo(CustomerCommissionInfoUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateCustomerCommissionInfoExists(updateReqVO.getId());
        // 更新
        CustomerCommissionInfoDO updateObj = CustomerCommissionInfoConvert.INSTANCE.convert(updateReqVO);
        customerCommissionInfoMapper.updateById(updateObj);
    }

    @Override
    public void deleteCustomerCommissionInfo(Long id) {
        // 校验存在
        this.validateCustomerCommissionInfoExists(id);
        // 删除
        customerCommissionInfoMapper.deleteById(id);
    }

    private void validateCustomerCommissionInfoExists(Long id) {
        if (customerCommissionInfoMapper.selectById(id) == null) {
            throw exception(CUSTOMER_COMMISSION_INFO_NOT_EXISTS);
        }
    }

    @Override
    public CustomerCommissionInfoDO getCustomerCommissionInfo(Long id) {
        return customerCommissionInfoMapper.selectById(id);
    }

    @Override
    public CustomerCommissionInfoDO getInfoByCommissionId(Long commissionId) {
        return customerCommissionInfoMapper.selectOne(CustomerCommissionInfoDO::getCommissionId, commissionId);
    }

    @Override
    public CustomerCommissionDto customerCommission(CustomerCommissionQueryDto customerCommissionQueryDto) {
        Long customerId = customerCommissionQueryDto.getCustomerId();
        if (customerId == null) {
            return new CustomerCommissionDto();
        }
        List<CustomerCommissionDO> commissionDOList = customerCommissionMapper.selectList(new LambdaQueryWrapperX<CustomerCommissionDO>().eq(CustomerCommissionDO::getCustomerId, customerId));
        if (CollectionUtil.isEmpty(commissionDOList)) {
            return new CustomerCommissionDto();
        }

        // 线路
        Integer transportId = null;
        Long departureId = null;
        Long objectiveId = null;

        Long lineId = customerCommissionQueryDto.getLineId();
        if (lineId != null) {
            LogisticsInfoDto logisticsInfoDto = warehouseLineMapper.getStartInfoAndDestInfoByLineId(lineId);
            if (logisticsInfoDto != null) {
                transportId = logisticsInfoDto.getTransportId();
                departureId = logisticsInfoDto.getStartCityId();
                objectiveId = logisticsInfoDto.getDestCityId();
            }
        }

        Integer finalTransportId = transportId;
        Long finalDepartureId = departureId;
        Long finalObjectiveId = objectiveId;

        //找到客户的佣金配置，并通过条件过滤出所有匹配的佣金配置
        if (lineId != null) {
            commissionDOList = commissionDOList.stream().filter(t ->
                    (t.getTransportId() == null && t.getDepartureId() == null && t.getObjectiveId() == null)
                            || (t.getTransportId().equals(ALL_INTEGER) && t.getDepartureId().equals(ALL_LONG) && t.getObjectiveId().equals(ALL_LONG))
                            || (t.getTransportId().equals(ALL_INTEGER) && t.getDepartureId().equals(ALL_LONG) && t.getObjectiveId().equals(finalObjectiveId))
                            || (t.getTransportId().equals(ALL_INTEGER) && t.getDepartureId().equals(finalDepartureId) && t.getObjectiveId().equals(finalObjectiveId))
                            || (t.getTransportId().equals(finalTransportId) && t.getDepartureId().equals(finalDepartureId) && t.getObjectiveId().equals(finalObjectiveId))
            ).collect(Collectors.toList());
        }
        Integer approval = customerCommissionQueryDto.getApproval();
        if (approval != null) {
            commissionDOList = commissionDOList.stream().filter(t -> t.getApproval() == null || t.getApproval().equals(ALL_INTEGER) || t.getApproval().equals(approval)).collect(Collectors.toList());
        }
        Integer customsType = customerCommissionQueryDto.getCustomsType();
        if (customsType != null) {
            commissionDOList = commissionDOList.stream().filter(t -> t.getCustomsType() == null || t.getCustomsType().equals(ALL_INTEGER) || t.getCustomsType().equals(customsType)).collect(Collectors.toList());
        }

        if (CollectionUtil.isEmpty(commissionDOList)) {
            return new CustomerCommissionDto();
        }
        //找到佣金详情
        List<CustomerCommissionInfoDO> commissionInfoDOList = customerCommissionInfoMapper.selectByCustomerId(customerId);
        if (CollectionUtil.isNotEmpty(commissionInfoDOList)) {
            //根据佣金过滤一遍佣金详情
            List<Long> commissionIdList = commissionDOList.stream().map(CustomerCommissionDO::getId).collect(Collectors.toList());
            commissionInfoDOList = commissionInfoDOList.stream().filter(info -> commissionIdList.contains(info.getCommissionId())).collect(Collectors.toList());

            //产品
            String[] productAttr = null;
            Long productType = null;
            Long productId = customerCommissionQueryDto.getProductId();
            if (productId != null) {
                ProductRespVO product = productApi.getById(productId);
                if (product != null) {
                    productAttr = StringUtils.isBlank(product.getAttrId()) ? null : product.getAttrId().split(StrUtil.COMMA);
                    productType = product.getTypeId();
                }
            }
            List<Long> finalProductAttr = Objects.nonNull(productAttr) ? Arrays.stream(productAttr).map(Long::parseLong).collect(Collectors.toList()) : Collections.singletonList(ALL_LONG);
            Long finalProductType = productType;

            //若是暗佣的话根据产品属性进行过滤
            if (productId != null) {
                commissionInfoDOList = commissionInfoDOList.stream().filter(t -> (t.getProductType() == null && t.getProductAttr() == null)
                        || (t.getProductType().equals(ALL_LONG) && t.getProductAttr().equals(ALL_LONG))
                        || (t.getProductType().equals(ALL_LONG) && finalProductAttr.contains(t.getProductAttr()))
                        || (t.getProductAttr().equals(ALL_LONG) && t.getProductType().equals(finalProductType))
                        || (t.getProductType().equals(finalProductType) && finalProductAttr.contains(t.getProductAttr()))
                ).collect(Collectors.toList());
            }

            List<CustomerCommissionDto> resultList = new ArrayList<>();

            //运费货币和计量单位过滤
            Integer freightCurrencyId = customerCommissionQueryDto.getFreightCurrencyId();
            Long freightVolumeId = customerCommissionQueryDto.getFreightVolumeId();
            if (freightCurrencyId != null && freightVolumeId != null) {

                for (CustomerCommissionInfoDO info : commissionInfoDOList) {
                    CustomerCommissionDO commissionTmp = commissionDOList.stream().filter(c -> c.getId().equals(info.getCommissionId())).findFirst().get();

                    //明佣
                    if (CommissionTypeEnum.LIGHT.getValue().equals(commissionTmp.getType())
                            && freightCurrencyId.equals(info.getRefundCurrency()) && Integer.valueOf(freightVolumeId.intValue()).equals(info.getRefundUnit())) {
                        resultList.add(new CustomerCommissionDto(commissionTmp.getType(), info.getRefund(), null));
                    }
                    //暗佣 - 产品
                    else if (CommissionTypeEnum.DARK.getValue().equals(commissionTmp.getType())
                            && info.getProductType() != null && info.getProductAttr() != null
                            && freightCurrencyId.equals(info.getDarkCurrency()) && Integer.valueOf(freightVolumeId.intValue()).equals(info.getDarkUnit())) {
                        resultList.add(new CustomerCommissionDto(commissionTmp.getType(), null, info.getDarkCommission()));
                    }
                    //暗佣 - 满返
                    else if (CommissionTypeEnum.DARK.getValue().equals(commissionTmp.getType())
                            && info.getProductType() == null && info.getProductAttr() == null
                            //满
                            && freightCurrencyId.equals(info.getAmountCurrency()) && Integer.valueOf(freightVolumeId.intValue()).equals(info.getAmountUnit())
                            //返
                            && freightCurrencyId.equals(info.getRefundCurrency()) && Integer.valueOf(freightVolumeId.intValue()).equals(info.getRefundUnit())) {

                        //满了就返
                        if (customerCommissionQueryDto.getOrgFreight() != null && customerCommissionQueryDto.getOrgFreight().compareTo(info.getAmount()) >= 0) {
                            resultList.add(new CustomerCommissionDto(commissionTmp.getType(), null, info.getRefund()));
                        }
                    }
                    //暗佣 + 明佣
                    else if (CommissionTypeEnum.DARK_WITH_LIGHT.getValue().equals(commissionTmp.getType())
                            //暗佣佣金
                            && freightCurrencyId.equals(info.getRefundCurrency()) && Integer.valueOf(freightVolumeId.intValue()).equals(info.getRefundUnit())
                            //销售价上调
                            && freightCurrencyId.equals(info.getDarkAdjustCurrency()) && Integer.valueOf(freightVolumeId.intValue()).equals(info.getDarkAdjustUnit())) {

                        resultList.add(new CustomerCommissionDto(commissionTmp.getType(), info.getDarkAdjustCommission(), info.getRefund()));
                    }

                }

            }


            //计算最优
            if (CollectionUtil.isNotEmpty(resultList)) {
                CustomerCommissionDto result = new CustomerCommissionDto();

                BigDecimal maxCommission = BigDecimal.ZERO;
                for (CustomerCommissionDto customerCommissionDto : resultList) {

                    if (CommissionTypeEnum.LIGHT.getValue().equals(customerCommissionDto.getCommissionType())) {
                        if (maxCommission.compareTo(customerCommissionDto.getPlainCommission()) < 1) {
                            maxCommission = customerCommissionDto.getPlainCommission();

                            result = customerCommissionDto;
                        }
                    } else if (CommissionTypeEnum.DARK.getValue().equals(customerCommissionDto.getCommissionType())) {

                        if (maxCommission.compareTo(customerCommissionDto.getHiddenCommission()) < 1) {
                            maxCommission = customerCommissionDto.getHiddenCommission();

                            result = customerCommissionDto;
                        }

                    } else if (CommissionTypeEnum.DARK_WITH_LIGHT.getValue().equals(customerCommissionDto.getCommissionType())) {

                        BigDecimal subtract = customerCommissionDto.getPlainCommission().add(customerCommissionDto.getHiddenCommission());
                        if (maxCommission.compareTo(subtract) < 1) {
                            maxCommission = subtract;

                            result = customerCommissionDto;
                        }

                    }

                }

                return result;
            }
        }

        return new CustomerCommissionDto();
    }

    @Override
    public List<CustomerCommissionInfoDO> getCustomerCommissionInfoList(Collection<Long> ids) {
        return customerCommissionInfoMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<CustomerCommissionInfoDO> getCustomerCommissionInfoPage(CustomerCommissionInfoPageReqVO pageReqVO) {
        return customerCommissionInfoMapper.selectPage(pageReqVO);
    }

    @Override
    public List<CustomerCommissionInfoDO> getCustomerCommissionInfoList(CustomerCommissionInfoExportReqVO exportReqVO) {
        return customerCommissionInfoMapper.selectList(exportReqVO);
    }

}
