package cn.iocoder.yudao.module.customer.service.customerFollowup;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.*;
import javax.annotation.Resource;

import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.CustomerDO;
import cn.iocoder.yudao.module.customer.dal.mysql.customer.CustomerMapper;
import cn.iocoder.yudao.module.customer.service.customerOperateLog.CustomerOperateLogService;
import cn.iocoder.yudao.module.customer.vo.customerOperateLog.CustomerOperateLogCreateReqVO;
import cn.iocoder.yudao.module.ecw.enums.CustomerFollowupStatusEnum;
import cn.iocoder.yudao.module.ecw.enums.CustomerOperateTypeEnum;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.customer.vo.customerFollowup.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerFollowup.CustomerFollowupDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.customer.convert.customerFollowup.CustomerFollowupConvert;
import cn.iocoder.yudao.module.customer.dal.mysql.customerFollowup.CustomerFollowupMapper;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.customer.enums.ErrorCodeConstants.*;

/**
 * 客户跟进 Service 实现类
 *
 * @author yanghao
 */
@Service
@Validated
public class CustomerFollowupServiceImpl extends AbstractService<CustomerFollowupMapper, CustomerFollowupDO> implements CustomerFollowupService {

    @Resource
    private CustomerFollowupMapper followupMapper;


    @Resource
    private CustomerOperateLogService customerOperateLogService;

    @Resource
    private CustomerMapper customerMapper;

    /**
     * 跟进单编号生成方式：GJ+年份+六位数字，按年份累计
     * @return
     */
    @Override
    public String generateFollowupNumber() {

        // 获取当前时间的年份
        LocalDate now = LocalDate.now();
        int year = now.getYear();
        // 获取当前年份的初始化时间
        LocalDateTime startOfYear = LocalDateTime.of(year, 1, 1, 0, 0, 0, 0);

        Date from = Date.from(startOfYear.atZone(ZoneId.systemDefault()).toInstant());

        Long count = followupMapper.selectCount(new LambdaQueryWrapperX<CustomerFollowupDO>()
                .ge(CustomerFollowupDO::getCreateTime, from));

        return  String.format("GJ%d%06d", year, count + 1);
    }

    @Override
    public Long createFollowup(CustomerFollowupCreateReqVO createReqVO) {

        String number = generateFollowupNumber();
        createReqVO.setNumber(number);

        // 插入
        CustomerFollowupDO followup = CustomerFollowupConvert.INSTANCE.convert(createReqVO);
        followupMapper.insert(followup);

        Long customerId = createReqVO.getCustomerId();
        if (customerId != null) {
            CustomerDO customer = customerMapper.selectById(customerId);
            if (customer == null) {

                // 插入日志
                LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
                //保存客户捞取记录
                CustomerOperateLogCreateReqVO customerOperateLogCreateReqVO = new CustomerOperateLogCreateReqVO()
                        .setOperator(loginUser == null ? null : loginUser.getId())
                        .setOperatorName(loginUser == null ? null : loginUser.getNickname())
                        .setCustomerId(customer.getId())
                        .setNumber(customer.getNumber())
                        .setName(customer.getName())
                        .setNewCustomerService(customer.getCustomerService())
                        .setOldCustomerService(customer.getCustomerService())
                        .setOldEstimateEnterOpenSeaTime(customer.getEstimateEnterOpenSeaTime())
                        .setNewEstimateEnterOpenSeaTime(customer.getEstimateEnterOpenSeaTime())
                        .setOperateType(CustomerOperateTypeEnum.FOLLOWUP_CREATE.getValue())
                        .setRemark("新增跟进纪录" + (CustomerFollowupStatusEnum.UnCommited.getValue().equals(createReqVO.getStatus()) ? "" : "，并提交"));
                customerOperateLogService.createOperateLog(customerOperateLogCreateReqVO);
            }
        }

        // 返回
        return followup.getId();
    }

    @Override
    public void updateFollowup(CustomerFollowupUpdateReqVO updateReqVO) {
        // 校验存在
        Long id = updateReqVO.getId();
        CustomerFollowupDO customerFollowupDO = followupMapper.selectById(id);
        if (customerFollowupDO == null) {
            throw exception(FOLLOWUP_NOT_EXISTS);
        }
        if (customerFollowupDO.getStatus().equals(CustomerFollowupStatusEnum.Commited.getValue())) {
            throw exception(FOLLOWUP_ALREADY_SUBMITTED);
        }
        // 更新
        CustomerFollowupDO updateObj = CustomerFollowupConvert.INSTANCE.convert(updateReqVO);
        followupMapper.updateById(updateObj);

        Long customerId = updateReqVO.getCustomerId();
        if (customerId != null) {
            CustomerDO customer = customerMapper.selectById(customerId);
            if (customer == null) {

                // 插入日志
                LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
                //保存客户捞取记录
                CustomerOperateLogCreateReqVO customerOperateLogCreateReqVO = new CustomerOperateLogCreateReqVO()
                        .setOperator(loginUser == null ? null : loginUser.getId())
                        .setOperatorName(loginUser == null ? null : loginUser.getNickname())
                        .setCustomerId(customer.getId())
                        .setNumber(customer.getNumber())
                        .setName(customer.getName())
                        .setNewCustomerService(customer.getCustomerService())
                        .setOldCustomerService(customer.getCustomerService())
                        .setOldEstimateEnterOpenSeaTime(customer.getEstimateEnterOpenSeaTime())
                        .setNewEstimateEnterOpenSeaTime(customer.getEstimateEnterOpenSeaTime())
                        .setOperateType(CustomerOperateTypeEnum.FOLLOWUP_UPDATE.getValue())
                        .setRemark("更新跟进纪录" + (CustomerFollowupStatusEnum.UnCommited.getValue().equals(updateReqVO.getStatus()) ? "" : "，并提交"));
                customerOperateLogService.createOperateLog(customerOperateLogCreateReqVO);
            }
        }
    }

    @Override
    public void updateFollowupStatus(CustomerFollowupUpdateStatusReqVO updateStatusReqVO) {

        Long id = updateStatusReqVO.getId();
        CustomerFollowupDO customerFollowupDO = followupMapper.selectById(id);
        if (customerFollowupDO == null) {
            throw exception(FOLLOWUP_NOT_EXISTS);
        }

        if (CustomerFollowupStatusEnum.Commited.getValue().equals(customerFollowupDO.getStatus())) {
            throw exception(FOLLOWUP_ALREADY_SUBMITTED);
        }

        customerFollowupDO.setStatus(updateStatusReqVO.getStatus());
        followupMapper.updateById(customerFollowupDO);



        Long customerId = customerFollowupDO.getCustomerId();
        if (customerId != null) {
            CustomerDO customer = customerMapper.selectById(customerId);
            if (customer == null) {

                // 插入日志
                LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
                //保存客户捞取记录
                CustomerOperateLogCreateReqVO customerOperateLogCreateReqVO = new CustomerOperateLogCreateReqVO()
                        .setOperator(loginUser == null ? null : loginUser.getId())
                        .setOperatorName(loginUser == null ? null : loginUser.getNickname())
                        .setCustomerId(customer.getId())
                        .setNumber(customer.getNumber())
                        .setName(customer.getName())
                        .setNewCustomerService(customer.getCustomerService())
                        .setOldCustomerService(customer.getCustomerService())
                        .setOldEstimateEnterOpenSeaTime(customer.getEstimateEnterOpenSeaTime())
                        .setNewEstimateEnterOpenSeaTime(customer.getEstimateEnterOpenSeaTime())
                        .setOperateType(CustomerOperateTypeEnum.FOLLOWUP_COMMIT.getValue())
                        .setRemark("提交跟进纪录");
                customerOperateLogService.createOperateLog(customerOperateLogCreateReqVO);
            }
        }
    }

    @Override
    public void deleteFollowup(Long id) {
        // 校验存在
        this.validateFollowupExists(id);
        // 删除
        followupMapper.deleteById(id);
    }

    private void validateFollowupExists(Long id) {
        if (followupMapper.selectById(id) == null) {
            throw exception(FOLLOWUP_NOT_EXISTS);
        }
    }

    @Override
    public CustomerFollowupDO getFollowup(Long id) {
        return followupMapper.selectById(id);
    }

    @Override
    public List<CustomerFollowupDO> getFollowupList(Collection<Long> ids) {
        return followupMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<CustomerFollowupDO> getFollowupPage(CustomerFollowupQueryVO query, PageVO page) {

        page.setPage(query.getPageNo());
        page.setRows(query.getPageSize());

        IPage<CustomerDO> mpPage = MyBatisUtils.buildPage(page);
        log.warn(I18nMessage.getLang().toString());
        int start = (page.getPage() - 1) * page.getRows();
        int size = page.getRows();
        List<CustomerFollowupDO> list = followupMapper.selectPage1(start, size, query);
        long total = followupMapper.selectCount1(query);
        return new PageResult<>(list, total,
                mpPage.getSize(), page.getPage(),
                (total + mpPage.getSize() - 1) / mpPage.getSize());
    }

    @Override
    public List<CustomerFollowupDO> getFollowupList(CustomerFollowupQueryVO query) {
        return followupMapper.selectPage1(null, null, query);
    }
}
